#include <amxmodx>
#include <engine>
#include <fakemeta>
#include <hamsandwich>
#include <cstrike>
#include <fun>
#include <timer_controller>
#include <cvi>

#define TASK_ROUNDTIME 1111
#define OFFSET_LINUX_WEAPONS 4
#define OFFSET_LINUX_PLAYERS 5

new const m_rgpPlayerItems_CWeaponBox[6] = {34, 35, ...};
new Trie:g_tCvarsToFile, g_iTrieSize;
new Float:g_fRoundTime, Float:g_fRoundStart, Float:g_fFreezeTime;
new g_pGameModeForward, g_pUpgradeClassForward;
new g_iPlayerData[33][PLAYER_DATA], g_iGlobalInfo[GLOBAL_INFO];
new cvar_preparation_time, cvar_roundtime;

public plugin_init()
{
  register_plugin("Cowboys vs Indians", CVI_VERSION, CVI_AUTHOR);
  register_cvar("cvi_server_version", CVI_VERSION, FCVAR_SERVER|FCVAR_EXTDLL|FCVAR_SPONLY);

  cvar_preparation_time = mynew_register_cvar("cvi_preparation_time", "1", "Preparation time before game start. (Default: 1)");
  cvar_roundtime        = mynew_register_cvar("cvi_roundtime", "5", "Round time. (Default: 5)");

  register_event("TextMsg", "Event_RestartRound", "a", "2&#Game_C", "2&#Game_w");
  register_event("HLTV", "Event_NewRound", "a", "1=0", "2=0");
  register_logevent("Event_StartRound", 2, "1=Round_Start");
  register_logevent("Event_EndRound", 2, "1=Round_End");

  RegisterHam(Ham_Touch, "weaponbox", "Ham_Touch_pre", 0);

  g_pGameModeForward = CreateMultiForward("cvi_gamemode", ET_IGNORE, FP_CELL);
  g_pUpgradeClassForward = CreateMultiForward("cvi_upgrade_class", ET_IGNORE, FP_CELL, FP_CELL, FP_CELL);
  register_dictionary("CowboysVSIndians.txt");
}

public plugin_natives()
{
  register_library("cvi");
  register_native("cvi_get_playerdata", "_get_playerdata");
  register_native("cvi_set_playerdata", "_set_playerdata");
  register_native("cvi_get_globalinfo", "_get_globalinfo");
  register_native("cvi_set_globalinfo", "_set_globalinfo");
  register_native("cvi_get_roundtime", "_get_roundtime");
  register_native("cvi_register_cvar", "_register_cvar");
}

public plugin_cfg()
{
  auto_exec_config(CVI_CONFIGFILE);
  g_fRoundTime = get_pcvar_float(cvar_roundtime);
  TrieDestroy(g_tCvarsToFile);
}

public client_disconnect(id)
{
  reset_user_one(id);
}

public client_putinserver(id)
{
  reset_user_one(id);
  set_task(11.0, "startup_info", id);
}

public startup_info(id)
{
  // Please don't remove :)
  client_print_color(id, print_team_default, "%s Mod created by ^3%s^1, ^4skype:guskis1^1, version: ^3%s^1!", CVI_TAG, CVI_AUTHOR, CVI_VERSION);
}

public Event_RestartRound()
{
  if(get_game_mode() != GAME_RESTARTING)
  {
    reset_user_all();
    reset_global_info();
    set_game_mode(GAME_RESTARTING);
    set_global_info(GI_ROUNDCOUNT, false);
  }
}

public Event_EndRound()
{
  if(get_game_mode() != GAME_ENDED)
  {
    set_game_mode(GAME_ENDED);
  }
}

public Event_NewRound()
{
  if(get_game_mode() != GAME_PREPARING)
  {
    new cvar = get_pcvar_num(cvar_preparation_time);
    if(!cvar) set_pcvar_num(cvar_preparation_time, cvar = 1);
    g_fFreezeTime = float(cvar);
    set_task(0.1, "set_timer");

    g_fRoundTime = get_pcvar_float(cvar_roundtime);
    g_fRoundStart = get_gametime();

    set_global_info(GI_ROUNDCOUNT, get_global_info(GI_ROUNDCOUNT)+1);
    reset_user_all();
    reset_global_info();
    set_game_mode(GAME_PREPARING);
  }
}

public set_timer()
  RoundTimerSet(0, get_pcvar_num(cvar_preparation_time));

public Event_StartRound()
{
  if(task_exists(TASK_ROUNDTIME))
    remove_task(TASK_ROUNDTIME);

  set_task(get_pcvar_float(cvar_preparation_time), "do_the_magic", TASK_ROUNDTIME);
}

public do_the_magic()
{
  RoundTimerSet(floatround(g_fRoundTime));

  new num, id, player_class;
  static players[32];
  get_players(players, num);

  for(--num; num >= 0; num--)
  {
    id = players[num];
    set_player_data(id, PD_CLASS, player_class = _:cs_get_user_team(id));
    client_print_color(id, print_team_default, "%s %L", CVI_TAG, id, "CVI_TEAM_SELECT",
      id, player_class == PC_COWBOY ? "CVI_COWBOY" : "CVI_INDIAN");
    set_user_godmode(id, 0);
  }

  set_game_mode(GAME_STARTED);
}


public Ham_Touch_pre(ent, id)
{
  if(get_game_mode() != GAME_STARTED)
    return HAM_SUPERCEDE;

  if(is_valid_ent(ent) && is_user_alive(id))
  {
    if(blockable_weapons(cs_get_weapon_id(weapon_in_box(ent)), get_player_data(id, PD_CLASS)))
      return HAM_SUPERCEDE;
  }

  return HAM_IGNORED;
}


stock weapon_in_box(ent)
{
  for(new weapon, i = 1; i < 6; i++)
  {
    weapon = get_pdata_cbase(ent, m_rgpPlayerItems_CWeaponBox[i], OFFSET_LINUX_WEAPONS);
    if(weapon > 0)
      return weapon;
  }
  
  return 0;
}

stock blockable_weapons(weapon_index, class)
{
  switch(class)
  {
    case PC_COWBOY:
    {
      if(weapon_index == CSW_M3 || weapon_index == CSW_DEAGLE)
        return 0;
    }
    case PC_INDIAN:
    {
      if(weapon_index == CSW_SCOUT || weapon_index == CSW_FIVESEVEN || weapon_index == CSW_SMOKEGRENADE)
        return 0;
    }
  }

  return 1;
}

stock get_game_mode()
  return get_global_info(GI_GAMEMODE);

stock set_game_mode(value)
{
  if(get_game_mode() && value == GAME_RESTARTING)
    log_amx("[CVI] Game has restarted :(");

  set_global_info(GI_GAMEMODE, value);
  new ret;
  ExecuteForward(g_pGameModeForward, ret, value);
}

stock get_player_data(id, pd)
  return g_iPlayerData[id][pd];

stock set_player_data(id, pd, value)
{
  if(pd == PD_UPGRADE)
  {
    new player_class = get_player_data(id, PD_CLASS);
    if(value)
      client_print_color(id, print_team_default, "%s %L", CVI_TAG, id, "CVI_UPGRADE",
        id, player_class == PC_COWBOY ? "CVI_COWBOY_U" : "CVI_INDIAN_U");
    new ret;
    ExecuteForward(g_pUpgradeClassForward, ret, id, player_class, value);
  }
  g_iPlayerData[id][pd] = value;
}

stock get_global_info(gi)
  return g_iGlobalInfo[gi];

stock set_global_info(gi, value)
  g_iGlobalInfo[gi] = value;

stock reset_user_all()
{
  new num, id;
  static players[32];
  get_players(players, num);

  for(--num; num >= 0; num--)
  {
    id = players[num];
    reset_user_one(id);
  }
}

stock reset_user_one(id)
{
  for(new i = 0; i < PLAYER_DATA; i++)
    set_player_data(id, i, false);

  if(task_exists(id))
    remove_task(id);
}

stock reset_global_info()
{
  if(task_exists(TASK_ROUNDTIME))
    remove_task(TASK_ROUNDTIME);

  for(new i = 0; i < GLOBAL_INFO; i++)
  {
    if(i == GI_ROUNDCOUNT) continue;
    set_global_info(i, false);
  }
}

Float:get_roundtime()
  return get_gametime() - g_fRoundStart - g_fFreezeTime;
  
stock mynew_register_cvar(name[], string[], description[], flags = 0, Float:fvalue = 0.0)
{
  new_register_cvar(name, string, description);
  return register_cvar(name, string, flags, fvalue);
}

stock new_register_cvar(name[], string[], description[], plug[] = "cvi_main.amxx")
{
  static path[96];
  if(!path[0])
  {
    get_localinfo("amxx_configsdir", path, charsmax(path));
    format(path, charsmax(path), "%s/%s", path, CVI_CONFIGFILE);
  }

  new file;
  if(!g_tCvarsToFile)
    g_tCvarsToFile = TrieCreate();

  if(!file_exists(path))
  {
    file = fopen(path, "wt");
    if(!file)
      return 0;

    fprintf(file, "// Server specific.^n");
    fprintf(file, "%-32s %-8s // %-32s // %s^n", "mp_tkpunish", "0", plug, "Disables TeamKill punishments");
    fprintf(file, "%-32s %-8s // %-32s // %s^n", "mp_friendlyfire", "0", plug, "Enables friendly fire to attack teamnates");
    fprintf(file, "%-32s %-8s // %-32s // %s^n", "mp_limitteams", "1", plug, "Disables team limits");
    fprintf(file, "%-32s %-8s // %-32s // %s^n", "mp_autoteambalance", "0", plug, "Disables team limits");
    fprintf(file, "%-32s %-8s // %-32s // %s^n", "mp_freezetime", "0", plug, "Disables freeze time on round start");
    fprintf(file, "%-32s %-8s // %-32s // %s^n", "mp_playerid", "2", plug, "Disables team info when aiming on player");
    fprintf(file, "%-32s %-8s // %-32s // %s^n", "sv_alltalk", "1", plug, "Enables alltalk");
    fprintf(file, "%-26s %-14s // %-32s // %s^n", "amx_statscfg", "off PlayerName", plug, "Disables player name when aiming on player");
    fprintf(file, "^n");
    fprintf(file, "// Mod specific.^n");
  }
  else
  {
    file = fopen(path, "rt");
    if(!file)
      return 0;

    //if(!TrieGetSize(g_tCvarsToFile))
    if(!g_iTrieSize)
    {
      new newline[48];
      static line[128];
      while(!feof(file))
      {
        fgets(file, line, charsmax(line));
        if(line[0] == ';' || !line[0])
          continue;

        parse(line, newline, charsmax(newline));
        remove_quotes(newline);
        #if AMXX_VERSION_NUM >= 183
          TrieSetCell(g_tCvarsToFile, newline, 1, false);
        #else
          TrieSetCell(g_tCvarsToFile, newline, 1);
        #endif
        g_iTrieSize++;
      }
    }
    fclose(file);
    file = fopen(path, "at");
  }

  if(!TrieKeyExists(g_tCvarsToFile, name))
  {
    fprintf(file, "%-32s %-8s // %-32s // %s^n", name, string, plug, description);
    #if AMXX_VERSION_NUM >= 183
      TrieSetCell(g_tCvarsToFile, name, 1, false);
    #else
      TrieSetCell(g_tCvarsToFile, name, 1);
    #endif
    g_iTrieSize++;
  }

  fclose(file);
  return 1;
}

// API
public _get_playerdata(plugin, params)
{
  if(params != 2)
    return -1;

  return get_player_data(get_param(1), get_param(2));
}

public _set_playerdata(plugin, params)
{
  if(params != 3)
    return -1;

  set_player_data(get_param(1), get_param(2), get_param(3));
  return 1;
}

public _get_globalinfo(plugin, params)
{
  if(params != 1)
    return -1;

  return get_global_info(get_param(1));
}

public _set_globalinfo(plugin, params)
{
  if(params != 2)
    return -1;

  set_global_info(get_param(1), get_param(2));
  return 1;
}

public _get_gamemode()
  return get_game_mode();

public _set_gamemode(plugin, params)
{
  if(params != 1)
    return -1;

  set_game_mode(get_param(1));
  return get_param(1);
}

public Float:_get_roundtime()
  return get_roundtime();

public _register_cvar(plugin, params)
{
	if(params != 3)
		return -1;

	static name[48], string[16], pluginname[48], description[128];
	get_string(1, name, charsmax(name));
	get_string(2, string, charsmax(string));
	get_string(3, description, charsmax(description));
	get_plugin(plugin, pluginname, charsmax(pluginname));

	return new_register_cvar(name, string, description, pluginname);
}
